@@ -1,24 +1,19 @@
 from __future__ import annotations
 
-import logging
 import re
 from collections.abc import Callable
 from typing import TYPE_CHECKING
 from urllib.parse import urlparse
 
-from plain.logs.utils import log_response
+from plain.exceptions import SuspiciousOperation
+from plain.http import HttpMiddleware
 from plain.runtime import settings
 
-from .views import CsrfFailureView
-
 if TYPE_CHECKING:
-    from plain.http import Response
-    from plain.http.request import Request
-
-logger = logging.getLogger("plain.security.csrf")
+    from plain.http import Request, Response
 
 
-class CsrfViewMiddleware:
+class CsrfViewMiddleware(HttpMiddleware):
     """
     Modern CSRF protection middleware using Sec-Fetch-Site headers and origin validation.
     Based on Filippo Valsorda's 2025 research (https://words.filippo.io/csrf/).
@@ -28,20 +23,20 @@ class CsrfViewMiddleware:
     """
 
     def __init__(self, get_response: Callable[[Request], Response]):
-        self.get_response = get_response
+        super().__init__(get_response)
 
         # Compile CSRF exempt patterns once for performance
         self.csrf_exempt_patterns: list[re.Pattern[str]] = [
             re.compile(r) for r in settings.CSRF_EXEMPT_PATHS
         ]
 
-    def __call__(self, request: Request) -> Response:
+    def process_request(self, request: Request) -> Response:
         allowed, reason = self.should_allow_request(request)
 
-        if allowed:
-            return self.get_response(request)
-        else:
-            return self.reject(request, reason)
+        if not allowed:
+            raise SuspiciousOperation(reason)
+
+        return self.get_response(request)
 
     def should_allow_request(self, request: Request) -> tuple[bool, str]:
         # 1. Allow safe methods (GET, HEAD, OPTIONS)
@@ -127,17 +122,3 @@ class CsrfViewMiddleware:
             False,
             f"Cross-origin request detected - Origin {origin} does not match Host",
         )
-
-    def reject(self, request: Request, reason: str) -> Response:
-        """Reject a request with a 403 Forbidden response."""
-
-        response = CsrfFailureView.as_view()(request, reason=reason)
-        log_response(
-            "Forbidden (%s): %s",
-            reason,
-            request.path,
-            response=response,
-            request=request,
-            logger=logger,
-        )
-        return response
