@@ -1,4 +1,7 @@
+from __future__ import annotations
+
 from pprint import pformat
+from typing import Any, NoReturn
 
 from markupsafe import Markup, escape
 
@@ -6,23 +9,24 @@ from plain.http import Response
 from plain.views.exceptions import ResponseException
 
 
-def dd(*objs):
+def dd(*objs: Any) -> NoReturn:
     """
     Dump and die.
 
     Dump the object and raise a ResponseException with the dump as the response content.
     """
 
-    print(f"Dumping objects:\n{'\n'.join([pformat(obj) for obj in objs])}")
+    print_objs = "\n".join([pformat(obj) for obj in objs])
+    print(f"Dumping objects:\n{print_objs}")
 
-    dump_strs = [
+    html_objs = [
         Markup("<pre><code>") + escape(pformat(obj)) + Markup("</code></pre>")
         for obj in objs
     ]
-    combined_dump_str = Markup("\n\n").join(dump_strs)
+    combined_dump_str = Markup("\n\n").join(html_objs)
 
     response = Response()
     response.status_code = 500
     response.content = combined_dump_str
-    response.content_type = "text/html"
+    response.headers["Content-Type"] = "text/html"
     raise ResponseException(response)
