@@ -1,6 +1,5 @@
 from __future__ import annotations
 
-import logging
 import types
 from typing import TYPE_CHECKING
 
@@ -8,7 +7,6 @@ from opentelemetry import baggage, trace
 from opentelemetry.semconv.attributes import http_attributes, url_attributes
 
 from plain.exceptions import ImproperlyConfigured
-from plain.logs.utils import log_response
 from plain.runtime import settings
 from plain.urls import get_resolver
 from plain.utils.module_loading import import_string
@@ -21,8 +19,6 @@ if TYPE_CHECKING:
     from plain.http import Request, Response, ResponseBase
     from plain.urls import ResolverMatch
 
-logger = logging.getLogger("plain.request")
-
 
 # These middleware classes are always used by Plain.
 BUILTIN_BEFORE_MIDDLEWARE = [
@@ -66,7 +62,7 @@ class BaseHandler:
                     f"Middleware factory {middleware_path} returned None."
                 )
 
-            handler = convert_exception_to_response(mw_instance)
+            handler = convert_exception_to_response(mw_instance.process_request)
 
         # We only assign to this when initialization is complete as it is used
         # as a flag for initialization being complete.
@@ -117,14 +113,9 @@ class BaseHandler:
                 else trace.StatusCode.ERROR
             )
 
-            if response.status_code >= 400:
-                log_response(
-                    "%s: %s",
-                    response.reason_phrase,
-                    request.path,
-                    response=response,
-                    request=request,
-                )
+            if response.exception:
+                span.record_exception(response.exception)
+
             return response
 
     def _get_response(self, request: Request) -> ResponseBase:
