@@ -7,13 +7,12 @@ def configure_logging(
     *, plain_log_level: int | str, app_log_level: int | str, app_log_format: str
 ) -> None:
     # Create and configure the plain logger (uses standard Logger, not AppLogger)
-    plain_logger = logging.Logger("plain")
+    plain_logger = logging.getLogger("plain")
     plain_logger.setLevel(plain_log_level)
     plain_handler = logging.StreamHandler()
     plain_handler.setFormatter(logging.Formatter("[%(levelname)s] %(message)s"))
     plain_logger.addHandler(plain_handler)
     plain_logger.propagate = False
-    logging.root.manager.loggerDict["plain"] = plain_logger
 
     # Configure the existing app_logger
     from .loggers import app_logger
