@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from abc import ABC, abstractmethod
 
 
@@ -5,6 +7,6 @@ class PreflightCheck(ABC):
     """Base class for all preflight checks."""
 
     @abstractmethod
-    def run(self):
+    def run(self) -> list:
         """Must return a list of Warning/Error results."""
         raise NotImplementedError
