@@ -1,7 +1,6 @@
 from __future__ import annotations
 
 import json
-import sys
 from http import HTTPStatus
 from http.cookies import SimpleCookie
 from io import BytesIO, IOBase
@@ -14,7 +13,7 @@ from plain.internal.handlers.base import BaseHandler
 from plain.internal.handlers.wsgi import WSGIRequest
 from plain.json import PlainJSONEncoder
 from plain.runtime import settings
-from plain.signals import got_request_exception, request_started
+from plain.signals import request_started
 from plain.urls import get_resolver
 from plain.utils.encoding import force_bytes
 from plain.utils.functional import SimpleLazyObject
@@ -55,7 +54,6 @@ class ClientResponse:
         response: ResponseBase,
         client: Client,
         request: dict[str, Any],
-        exc_info: tuple[Any, Any, Any] | None,
     ):
         # Store wrapped response in __dict__ directly to avoid __setattr__ recursion
         object.__setattr__(self, "_response", response)
@@ -66,7 +64,6 @@ class ClientResponse:
         self.wsgi_request: WSGIRequest
         self.redirect_chain: list[tuple[str, int]]
         self.resolver_match: SimpleLazyObject | ResolverMatch
-        self.exc_info = exc_info
         # Optional: set by plain.auth if available
         # self.user: Model
 
@@ -530,7 +527,6 @@ class Client:
         self._request_factory = RequestFactory(headers=headers, **defaults)
         self.handler = ClientHandler()
         self.raise_request_exception = raise_request_exception
-        self.exc_info: tuple[Any, Any, Any] | None = None
         self.extra: dict[str, Any] | None = None
         self.headers: dict[str, str] | None = None
 
@@ -553,25 +549,20 @@ class Client:
         """
         environ = self._request_factory._base_environ(**request)
 
-        # Capture exceptions created by the handler.
-        exception_uid = f"request-exception-{id(request)}"
-        got_request_exception.connect(self.store_exc_info, dispatch_uid=exception_uid)
-        try:
-            response = self.handler(environ)
-        finally:
-            # signals.template_rendered.disconnect(dispatch_uid=signal_uid)
-            got_request_exception.disconnect(dispatch_uid=exception_uid)
+        # Make the request
+        response = self.handler(environ)
 
         # Wrap the response in ClientResponse for test-specific attributes
         client_response = ClientResponse(
             response=response,
             client=self,
             request=request,
-            exc_info=self.exc_info,
         )
 
-        # Check for signaled exceptions and potentially re-raise
-        self.check_exception()
+        # Re-raise the exception if configured to do so
+        # Only 5xx errors have response.exception set
+        if client_response.exception and self.raise_request_exception:
+            raise client_response.exception
 
         # If the request had a user, make it available on the response.
         try:
@@ -901,18 +892,6 @@ class Client:
 
         return response
 
-    def store_exc_info(self, **kwargs: Any) -> None:
-        """Store exceptions when they are generated by a view."""
-        self.exc_info = sys.exc_info()
-
-    def check_exception(self) -> None:
-        """Check for signaled exceptions and potentially re-raise."""
-        if self.exc_info:
-            _, exc_value, _ = self.exc_info
-            self.exc_info = None
-            if self.raise_request_exception:
-                raise exc_value
-
     @property
     def session(self) -> Any:
         """Return the current session variables."""
